<?php
/**
 * Copyright  2016 Filoblu S.r.l. All rights reserved.
 */ 
namespace FiloBlu\Esb\Helper\Format;

/**
 * Factory class to build Format Helper depending on System Configuration (JSON or XML)
 */

class FormatFactory{
    /**
     * @var array
     */
    protected $_pluginConfiguration;

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $_scopeConfig;

    /**
     * @var \Magento\Framework\ObjectManagerInterface
     */
    protected $_objectManager;

    /**
     * Constructor
     *
     * @param \Magento\Framework\ObjectManagerInterface $objectManager
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig   
     */
    public function __construct(
        \Magento\Framework\ObjectManagerInterface $objectManager,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
    )
    {
        $this->_objectManager = $objectManager;
        $this->_scopeConfig = $scopeConfig;
        $this->_pluginConfiguration = array(
            "xml" => "\FiloBlu\Esb\Helper\Format\Xml",
            "json" => "\FiloBlu\Esb\Helper\Format\Json",
        );
    }

    /**
    * Creates Format Helper instance depending on System Configuration (JSON or XML)
    */    
    public function create(){
        $configuredFormat=$this->_scopeConfig->getValue(
            'esb/general/format'
        );
        if(array_key_exists($configuredFormat, $this->_pluginConfiguration)){
            $configuredClass=$this->_pluginConfiguration[$configuredFormat];
            $formatHelper = $this->_objectManager->create($configuredClass);
            if (false == $formatHelper instanceof FormatInterface) {
                throw new \Exception(
                    'Object is not instance of FormatInterface'
                );
            }
        }else{
            throw new \Exception("Cannot find format class for $configuredFormat");
        }        
        return $formatHelper;
    }
}
