<?php
/**
 * Copyright © 2016 Filoblu S.r.l. All rights reserved.
 */

namespace FiloBlu\Esb\Model\Rest;
use FiloBlu\Esb\Api\ExternalCreditmemoInterface;
use Magento\Framework\App\Filesystem\DirectoryList;
use Magento\Sales\Api\Data\CreditmemoInterface;
use Magento\Sales\Exception\CouldNotRefundException;

if (!defined("DS")):
      define("DS", DIRECTORY_SEPARATOR);
endif;

/**
 * REST Api to save external creditmemos
 */
class ExternalCreditmemo implements ExternalCreditmemoInterface{
	
    /**
     * @var \Magento\Sales\Api\Data\OrderExtensionFactory
     */ 	
    protected $_orderRepository;
	
    /**
     * @var \Magento\Framework\Api\SearchCriteriaBuilder 
     */ 
    protected $_searchCriteriaBuilder;

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */  
    protected $_scopeConfig;

    /**
     * @var \Magento\Sales\Api\Data\OrderExtensionFactory
     */ 
    protected $_orderExtensionFactory;

    /**
     * @var \Magento\Sales\Model\Order\CreditmemoFactory
     */ 
    protected $_creditmemoFactory;
            
    /**
     * @var \Magento\Sales\Model\Service\CreditmemoService
     */ 
    protected $_creditmemoService;
            
    /**
     * @var \Magento\Framework\Filesystem
     */ 
    protected $_filesystem;
    
    /**
     * @var \Monolog\Logger
     */ 
    protected $_logger;
  
    /**
     * @var \FiloBlu\Esb\Api\Data\AcknowledgeInterface
     */     
    protected $_response;
    
    /**
     * Init plugin
     *
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Magento\Sales\Model\OrderRepository $orderRepository
     * @param \Magento\Sales\Api\Data\OrderExtensionFactory $orderExtensionFactory
     * @param \Magento\Sales\Model\Order\CreditmemoFactory $creditmemoFactory
     * @param \Magento\Sales\Model\Service\CreditmemoService $creditmemoService
     * @param \Magento\Framework\Filesystem $filesystem
     * @param \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder     
     */
    public function __construct(
            \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
            \Magento\Sales\Model\OrderRepository $orderRepository,
            \Magento\Sales\Api\Data\OrderExtensionFactory $orderExtensionFactory,
            \Magento\Sales\Model\Order\CreditmemoFactory $creditmemoFactory,
            \Magento\Sales\Model\Service\CreditmemoService $creditmemoService,
            \Magento\Framework\Filesystem $filesystem,
            \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder,
            \Filoblu\Esb\Helper\LoggerProvider $loggerProvider,
            \FiloBlu\Esb\Api\Data\AcknowledgeInterface $response
    ){
        $this->_scopeConfig = $scopeConfig;
        $this->_orderRepository = $orderRepository;
        $this->_orderExtensionFactory = $orderExtensionFactory;
        $this->_creditmemoFactory = $creditmemoFactory;
        $this->_creditmemoService = $creditmemoService;
        $this->_filesystem = $filesystem;        
        $this->_searchCriteriaBuilder = $searchCriteriaBuilder;              
        $this->_logger=$loggerProvider->getLogger();
        $this->_response = $response;
    }

    /**
     * Save External Creditmemo data. Save PDF file and additional data on Sales Order.
     *
     * @param \Magento\Sales\Api\Data\CreditmemoInterface  $creditmemo_data
     * @return int
     */
     public function save(CreditmemoInterface $creditmemo_data){
        $creditmemoExtensionAttributes=$creditmemo_data->getExtensionAttributes();
        
        if(null==$creditmemoExtensionAttributes || !$creditmemoExtensionAttributes->getOrderIncrementId() || !$creditmemoExtensionAttributes->getDocumentType()){
            $message=__("Cannot find order to refund");
            $this->_logger->critical($message);
            throw new CouldNotRefundException($message);
        }
        if(null==$creditmemo_data->getIncrementId()){
            $message=__("Creditmemo increment id is missing.");
            $this->_logger->critical($message);
            throw new CouldNotRefundException($message);
        }
        $orderIncrementId=$creditmemoExtensionAttributes->getOrderIncrementId();
        $creditmemoIncrementId=$creditmemo_data->getIncrementId();
        $documentType=$creditmemoExtensionAttributes->getDocumentType();
        $filename="order_".$orderIncrementId."_".$documentType."_".$creditmemoIncrementId.".pdf";
        $filename=preg_replace("/[^0-9A-Za-z.]+/","_",$filename );
        $documentFolder = $this->_scopeConfig->getValue(
            'esb/print_documents/document_folder'
        );              
        $writer = $this->_filesystem->getDirectoryWrite(DirectoryList::VAR_DIR);
        /** @var \Magento\Framework\Filesystem\File\WriteInterface|\Magento\Framework\Filesystem\File\Write $file */
        
        $file = $writer->openFile($documentFolder.DS.$filename, 'w');
        try {
            $file->lock();
            try {
                $file->write(base64_decode($creditmemoExtensionAttributes->getDocumentPdf()));
            }
            finally {
                $file->unlock();
            }
        }
        finally {
            $file->close();
        }

        $refund_online=$creditmemoExtensionAttributes->getRefundOnline();
        $creditmemo=array(
            "increment_id" => $creditmemo_data->getIncrementId(),
            "document_type" => $creditmemoExtensionAttributes->getDocumentType(),
            "grand_total" => $creditmemo_data->getGrandTotal(),
            "document_pdf" => $filename,
            "refund_online" => $refund_online,
        );


        $searchCriteria = $this->_searchCriteriaBuilder
            ->addFilter('increment_id', $creditmemoExtensionAttributes->getOrderIncrementId(), 'eq')->create();
        $orderList = $this->_orderRepository->getList($searchCriteria);
        
        if(count($orderList)<>1){
            $message=__("Cannot find order %1", $creditmemoExtensionAttributes->getOrderIncrementId());
            $this->_logger->critical($message);
            throw new CouldNotRefundException($message);           
        }
        try{
            $order=$orderList->getFirstItem();
            if(!$order->canCreditmemo()){
                $message=__("Cannot create creditmemo for order %1", $creditmemoExtensionAttributes->getOrderIncrementId());
                $this->_logger->critical($message);
                throw new CouldNotRefundException($message);           
            }
            $qtys=array();
            $creditmemo_items=$creditmemo_data["items"];
            foreach($order->getAllVisibleItems() as $order_item){
                foreach($creditmemo_items as $key => $creditmemo_item){
                    if($creditmemo_item["sku"] == $order_item->getSku()){
                        $qtys[$order_item->getId()] = $creditmemo_item["qty"];
                        break;
                        //unset($creditmemo_items[$key]);
                    }
                }
            }
            $creditmemo_obj = $this->_creditmemoFactory->createByOrder(
                    $order, 
                    array(
                        "qtys" => $qtys,
                        "shipping_amount" => $creditmemo_data->getShippingInclTax()
                    )
            );   
            
            if($refund_online){
                $invoices = $order->getInvoiceCollection();     
                if($invoices->getSize() <>1){ 
                    $message=__("No invoices to refund.");
                    $this->_logger->critical($message);
                    throw new CouldNotRefundException($message);                    
                }
                $invoice_obj = $invoices->getFirstItem();
                $creditmemo_obj->setInvoice($invoice_obj);                   
                if(abs($creditmemo_obj->getGrandTotal() -  $creditmemo_data->getGrandTotal()) >0.1){
                    $message=__("Grand Total is not matching: %1 <> %2", $creditmemo_obj->getGrandTotal(), $creditmemo_data->getGrandTotal() );
                    $this->_logger->critical($message);
                    throw new CouldNotRefundException($message);
                }
                $this->_creditmemoService->refund($creditmemo_obj);
            }else{                
                $creditmemo_obj->save();
            }
            
            
            $orderExtensionAttributes=$order->getExtensionAttributes();
            $additionalData=array("external_creditmemos" => array($creditmemo_data->getIncrementId() => $creditmemo));
            if($orderExtensionAttributes){
                if($orderExtensionAttributes->getFbAdditionalData()){
                    $additionalData=current($orderExtensionAttributes->getFbAdditionalData());
                    if(array_key_exists("external_creditmemos", $additionalData)){
                        $additionalData["external_creditmemos"][$creditmemo_data->getIncrementId()]=$creditmemo;
                    }else{
                        $additionalData["external_creditmemos"]=array($creditmemo_data->getIncrementId() => $creditmemo);
                    }
                }
            }
            else{
                $orderExtensionAttributes=$this->_orderExtensionFactory->create();
            }
            $orderExtensionAttributes->setFbAdditionalData(array($additionalData));
            $order->setExtensionAttributes($orderExtensionAttributes);
            $order->addStatusHistoryComment(__("External refund has been saved: %1 %2", $documentType, $creditmemoIncrementId), "closed");
            $order->addStatusHistoryComment(__("Order %1 has been refunded", $order->getIncrementId()), "closed");
            $this->_orderRepository->save($order);
            
            $this->_response->setStatus("success");
            $this->_response->setCode($documentType."_".$creditmemoIncrementId);
            $this->_response->setMessage(__("Order %1 has been refunded:  %2 %3 has been created", $order->getIncrementId(), $documentType, $creditmemoIncrementId));
            $this->_response->setDescription(__("Magento Creditmemo: %1. Refunded Amount: ", $creditmemo_obj->getIncrementId()), $creditmemo_obj->getGrandTotal());
            $this->_response->setFlow("create_refund");            
            $this->_response->setOrderIncrementId($order->getIncrementId());            
            return $this->_response;
        }catch(Exception $e){
            $this->_logger->critical($e->getMessage());
            throw $e;
        }
    }
}

